// FileDialog.java
// $Id: FileDialog.java,v 1.3 1999/11/24 10:25:44 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
package org.w3c.jwput.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;

import org.w3c.tools.widgets.Utilities;

/**
 * Winie's Dialog used to get a Filename
 * @version $Revision: 1.3 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class FileDialog extends JPanel implements JWDialog {

    protected String title    = null;
    protected String value    = null;
    protected int returnValue = ERROR_OPTION;

    protected JDialog    dialog = null;
    protected JTextField tfile  = null;

    public FileDialog(String title, String message[]) {
	this.title = title;
	build(message);
    }

    protected void build(String message[]) {
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("Ok")) {
		    value = tfile.getText();
		    if (value.length() > 0) {
			returnValue = APPROVE_OPTION;
			done();
		    } else {
			value = null;
		    }
		} else if ( ae.getActionCommand().equals("Cancel")) {
		    value = null;
		    returnValue = CANCEL_OPTION;
		    done();
		}
	    }
	};

	ActionListener bl = new ActionListener() {
	    JFileChooser chooser = null;
	    public void actionPerformed(ActionEvent e) {
		if (chooser == null) {
		    chooser = new JFileChooser();
		    chooser.setMultiSelectionEnabled(true);
		    chooser.setFileSelectionMode(
					      JFileChooser.DIRECTORIES_ONLY);
		}
		int returnVal = 
		    chooser.showOpenDialog((JComponent)e.getSource());
		if (returnVal ==  JFileChooser.APPROVE_OPTION) {
		    tfile.setText(chooser.getSelectedFile().getAbsolutePath());
		}
	    }
	};

	GridBagLayout mgbl = new GridBagLayout();
	GridBagConstraints mgbc = new GridBagConstraints();
	setLayout(mgbl);

	tfile = new JTextField(40);
	tfile.setEditable(true);
	tfile.setBorder(BorderFactory.createLoweredBevelBorder());

	JButton browse = new JButton("Browse");
	browse.addActionListener(bl);

	JPanel p1 = new JPanel(new BorderLayout(4, 0));
	p1.add(tfile, "Center");
	p1.add(browse, "East");

	for (int i = 0 ; i < message.length ; i++) {
	    JLabel l = new JLabel(message[i], JLabel.LEFT);
	    l.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));
	    mgbc.gridwidth = GridBagConstraints.REMAINDER;
	    mgbc.anchor = GridBagConstraints.WEST;
	    mgbl.setConstraints(l, mgbc);
	    add(l);
	}

	GridBagLayout gbl = new GridBagLayout();
 	GridBagConstraints gbc = new GridBagConstraints();
 	gbc.fill = GridBagConstraints.HORIZONTAL;
 	gbc.weightx = 0.9;
 	gbc.weighty = 0;
 	gbc.insets = Utilities.insets4;
	JPanel destp = new JPanel(gbl);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p1, gbc);
	destp.add(p1);

	mgbc.gridwidth = GridBagConstraints.REMAINDER;
	mgbl.setConstraints(destp, mgbc);
	add(destp);

	// and now the usual button bar
	JPanel p = new JPanel(new GridLayout(1, 2, 20, 20));
	p.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	JButton b = new JButton("Ok");
	b.addActionListener(al);
	p.add(b);
	b = new JButton("Cancel");
	b.addActionListener(al);
	p.add(b);
	mgbc.anchor = GridBagConstraints.CENTER;
	mgbl.setConstraints(p, mgbc);
	add(p);
	setBorder(BorderFactory.createTitledBorder(title));
    }

    protected void done() {
	if (dialog != null)
	    dialog.setVisible(false);
    }

    /**
     * Show this modal dialog
     * @param comp the parent component
     * @return CANCEL_OPTION, APPROVE_OPTION, ERROR_OPTION
     */
    public int showDialog(Component parent) {
	Frame frame = (parent instanceof Frame)
	    ? (Frame) parent
	    : (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);

	dialog = new JDialog(frame, title, true);
	Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);

	dialog.pack();
        dialog.setLocationRelativeTo(parent);
        dialog.show();
	return returnValue;
    }

    /**
     * Get the string
     * @return a String instance
     */
    public String getFile() {
	return value;
    }

}

