// LanguageDialog.java
// $Id: LanguageDialog.java,v 1.3 1999/10/12 08:44:31 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
 
package org.w3c.jwput.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;


/**
 * Winie's Dialog use to select Languages
 * @version $Revision: 1.3 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class LanguageDialog extends JPanel implements JWDialog {

    protected static final String OK_B     = "  Ok  ";
    protected static final String CANCEL_B = "Cancel";
    protected static final String TEXT_B   = "text";

    protected String title    = null;
    protected String value    = null;

    protected int returnValue = ERROR_OPTION;

    protected JDialog        dialog = null;
    protected JTextField     otherT = null;
    protected JList          list   = null;

    /**
     * Constructor
     * @param the Dialog title
     */ 
    public LanguageDialog(String title) {
	this.title    = title;
	this.value    = null;
	build(title);
    }

    protected void build(String title) {
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals(OK_B)) {
		    value = otherT.getText();
		    if ((value == null) || (value.equals("")))
			value = (String)list.getSelectedValue();
		    returnValue = APPROVE_OPTION;
		    done();
		} else if ( ae.getActionCommand().equals(TEXT_B)) {
		    value = otherT.getText();
		    returnValue = APPROVE_OPTION;
		    done();
		} else if ( ae.getActionCommand().equals(CANCEL_B)) {
		    returnValue = CANCEL_OPTION;
		    done();
		} 
	    }
	};

	JLabel line = new JLabel("Select the language to add.");
	line.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	
	list = new JList(Languages.languages_array);
	//list.setBorder(BorderFactory.createLoweredBevelBorder());
	list.setFixedCellWidth(100);
	list.setVisibleRowCount(10);

	JLabel other = new JLabel("Other");
	otherT = new JTextField(10);
	otherT.setBorder(BorderFactory.createLoweredBevelBorder());
	otherT.addActionListener(al);
	otherT.setActionCommand(TEXT_B);

	JButton okb     = new JButton(OK_B);
	JButton cancelb = new JButton(CANCEL_B);
	okb.addActionListener(al);
	cancelb.addActionListener(al);

	JPanel p1 = new JPanel(new BorderLayout(5,5));
	p1.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	p1.add(new JScrollPane(list), "Center");
	p1.add(line, "North");

	JPanel p2 = new JPanel(new BorderLayout(5,5));
	p2.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	p2.add(other, "West");
	p2.add(otherT, "Center");

	JPanel p3 = new JPanel(new BorderLayout());
	p3.add(p1, "Center");
	p3.add(p2, "South");

	JPanel p4 = new JPanel(new BorderLayout());
	p4.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	p4.add(okb, "West");
	p4.add(cancelb, "East");

	setLayout(new BorderLayout());
	add(p3, "Center");
	add(p4, "South");
    }

    protected void done() {
	if (dialog != null)
	    dialog.setVisible(false);
    }

    /**
     * Show this modal dialog
     * @param comp the parent component
     * @return CANCEL_OPTION, APPROVE_OPTION, ERROR_OPTION
     */
    public int showDialog(Component parent) {
	Frame frame = (parent instanceof Frame)
	    ? (Frame) parent
	    : (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);

	dialog = new JDialog(frame, title, true);
	Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);

	dialog.pack();
        dialog.setLocationRelativeTo(parent);
	dialog.setSize(250,400);
        dialog.show();
	return returnValue;
    }

    /**
     * Get the selected language
     * @return a String instance
     */
    public String getLanguage() {
	return value;
    }

}
